@@ -1,7 +1,7 @@
 package WWW::PAUSE::Simple;
 
-our $DATE = '2015-09-11'; # DATE
-our $VERSION = '0.28'; # VERSION
+our $DATE = '2016-01-17'; # DATE
+our $VERSION = '0.29'; # VERSION
 
 use 5.010001;
 use strict;
@@ -93,6 +93,15 @@ our %mod_opt_arg = (
     },
 );
 
+our %protect_files_arg = (
+    protect_files => {
+        summary => 'Protect some files/wildcard patterns from delete/cleanup',
+        schema  => ['array*', of=>'str*'],
+        'x.name.is_plural' => 1,
+        tags => ['category:filtering'],
+    },
+);
+
 $SPEC{':package'} = {
     v => 1.1,
     summary => 'An API for PAUSE',
@@ -152,7 +161,8 @@ $SPEC{upload_files} = {
         },
         delay => {
             summary => 'Pause a number of seconds between files',
-            schema => ['int*', min=>0],
+            schema => ['duration*'],
+            'x.perl.coerce_to' => 'int(secs)',
             description => <<'_',
 
 If you upload a lot of files (e.g. 7-10 or more) at a time, the PAUSE indexer
@@ -164,7 +174,8 @@ _
         },
         group_delay => {
             summary => 'Pause a number of seconds between groups of files',
-            schema => ['int*', min=>0],
+            schema => 'duration*',
+            'x.perl.coerce_to' => 'int(secs)',
             description => <<'_',
 
 As an alternative to the `delay` option, you can also use this option. This will
@@ -529,6 +540,7 @@ _
     args => {
         %common_args,
         %detail_l_arg,
+        %protect_files_arg,
         num_keep => {
             schema => ['int*', min=>1],
             default => 1,
@@ -559,8 +571,12 @@ sub delete_old_releases {
         $file =~ s/\.$re_archive_ext\z//;
         push @to_delete, "$file.*";
     }
-    $res = delete_files(_common_args(\%args),
-                        files=>\@to_delete, -dry_run=>$args{-dry_run});
+    $res = delete_files(
+        _common_args(\%args),
+        protect_files => $args{protect_files},
+        files=>\@to_delete,
+        -dry_run=>$args{-dry_run},
+    );
     return $res if $res->[0] != 200 || $args{-dry_run};
     my $deleted_files = $res->[3]{'func.files'} // [];
     if (@$deleted_files) {
@@ -582,16 +598,20 @@ sub _delete_or_undelete_or_reindex_files {
     my $files0 = $args{files} // [];
     return [400, "Please specify at least one file"] unless @$files0;
 
+    my $protect_files = $args{protect_files} // [];
+
     my @files;
     {
         my $listres;
+        if (grep {String::Wildcard::Bash::contains_wildcard($_)}
+                (@$files0, @$protect_files)) {
+            $listres = list_files(_common_args(\%args));
+            return [500, "Can't list files: $listres->[0] - $listres->[1]"]
+                unless $listres->[0] == 200;
+        }
+
         for my $file (@$files0) {
             if (String::Wildcard::Bash::contains_wildcard($file)) {
-                unless ($listres) {
-                    $listres = list_files(_common_args(\%args));
-                    return [500, "Can't list files: $listres->[0] - $listres->[1]"]
-                        unless $listres->[0] == 200;
-                }
                 my $re = Regexp::Wildcards->new(type=>'unix')->convert($file);
                 $re = qr/\A($re)\z/;
                 for my $f (@{$listres->[2]}) {
@@ -601,6 +621,32 @@ sub _delete_or_undelete_or_reindex_files {
                 push @files, $file;
             }
         }
+
+        for my $protect_file (@$protect_files) {
+            if (String::Wildcard::Bash::contains_wildcard($protect_file)) {
+                my $re = Regexp::Wildcards->new(type=>'unix')->convert(
+                    $protect_file);
+                $re = qr/\A($re)\z/;
+                @files = grep {
+                    if ($_ =~ $re) {
+                        $log->debugf("Excluding %s (protected, wildcard %s)",
+                                     $_, $protect_file);
+                        0;
+                    } else {
+                        1;
+                    }
+                } @files;
+            } else {
+                @files = grep {
+                    if ($_ eq $protect_file) {
+                        $log->debugf("Excluding %s (protected)", $_);
+                        0;
+                    } else {
+                        1;
+                    }
+                } @files;
+            }
+        }
     }
 
     unless (@files) {
@@ -646,6 +692,7 @@ _
     args => {
         %common_args,
         %files_arg,
+        %protect_files_arg,
     },
     features => {dry_run=>1},
 };
@@ -827,7 +874,7 @@ WWW::PAUSE::Simple - An API for PAUSE
 
 =head1 VERSION
 
-This document describes version 0.28 of WWW::PAUSE::Simple (from Perl distribution WWW-PAUSE-Simple), released on 2015-09-11.
+This document describes version 0.29 of WWW::PAUSE::Simple (from Perl distribution WWW-PAUSE-Simple), released on 2016-01-17.
 
 =head1 SYNOPSIS
 
@@ -847,6 +894,8 @@ When a file is deleted, it is not immediately deleted but has
 scheduled_for_deletion status for 72 hours, then deleted. During that time, the
 file can be undeleted.
 
+This function is not exported by default, but exportable.
+
 This function supports dry-run operation.
 
 
@@ -862,6 +911,10 @@ File names/wildcard patterns.
 
 PAUSE password.
 
+=item * B<protect_files> => I<array[str]>
+
+Protect some files/wildcard patterns from delete/cleanup.
+
 =item * B<username>* => I<str>
 
 PAUSE ID.
@@ -902,6 +955,8 @@ command line:
  % pause rm 'My-Module-*TRIAL*'; # delete a dist's trial releases
  % pause rm '*TRIAL*' '*_*'; # delete all files containing TRIAL or underscore
 
+This function is not exported by default, but exportable.
+
 This function supports dry-run operation.
 
 
@@ -924,6 +979,10 @@ second newest, and so on.
 
 PAUSE password.
 
+=item * B<protect_files> => I<array[str]>
+
+Protect some files/wildcard patterns from delete/cleanup.
+
 =item * B<username>* => I<str>
 
 PAUSE ID.
@@ -960,6 +1019,8 @@ Distribution names will be extracted from tarball/zip filenames.
 
 Unknown/unparseable filenames will be skipped.
 
+This function is not exported by default, but exportable.
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -1006,6 +1067,8 @@ Return value:  (any)
 
 List files.
 
+This function is not exported by default, but exportable.
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -1048,6 +1111,8 @@ Return value:  (any)
 
 List modules (permissions).
 
+This function is not exportable.
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -1090,6 +1155,8 @@ Return value:  (any)
 
 Force reindexing.
 
+This function is not exported by default, but exportable.
+
 This function supports dry-run operation.
 
 
@@ -1141,6 +1208,8 @@ When a file is deleted, it is not immediately deleted but has
 scheduled_for_deletion status for 72 hours, then deleted. During that time, the
 file can be undeleted.
 
+This function is not exported by default, but exportable.
+
 This function supports dry-run operation.
 
 
@@ -1188,6 +1257,8 @@ Return value:  (any)
 
 Upload file(s).
 
+This function is not exportable.
+
 This function supports dry-run operation.
 
 
@@ -1195,7 +1266,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<delay> => I<int>
+=item * B<delay> => I<duration>
 
 Pause a number of seconds between files.
 
@@ -1208,7 +1279,7 @@ files will alleviate this problem.
 
 File names/wildcard patterns.
 
-=item * B<group_delay> => I<int>
+=item * B<group_delay> => I<duration>
 
 Pause a number of seconds between groups of files.
 
@@ -1296,7 +1367,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by perlancar@cpan.org.
+This software is copyright (c) 2016 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
